﻿'====================================================================
'   pas1000w.vb
'
'   Copyright (C) P&A Technologies Inc.
'====================================================================

Option Strict Off
Option Explicit On

Imports System.Runtime.InteropServices

Public Class pas1000

    '----------------------------------------------------------------
    ' デバイスアクセスに関する定義
    '----------------------------------------------------------------
    Public Const MAX_DEVICE As UInteger = 16                        ' 最大デバイス数
    Public Const DEVICE_AUTO As UShort = &HFFFF                     ' デバイス検索指定

    '----------------------------------------------------------------
    ' チャンネルに関する定義
    '----------------------------------------------------------------
    Public Const ANALOG_NUM_CH As Integer = 8                       ' アナログチャンネル数
    Public Const EXTRIG_NUM_CH As Integer = 8                       ' 外部トリガチャンネル数
    Public Const DIGITALOUT_NUM_CH As Integer = 4                   ' デジタル出力チャンネル数

    '----------------------------------------------------------------
    ' デバイス情報に関する定義
    '----------------------------------------------------------------
    Public Const VID_LENGTH As Integer = 8                          ' ベンダ識別文字列長
    Public Const PID_LENGTH As Integer = 16                         ' プロダクト識別文字列長

    '----------------------------------------------------------------
    ' 時間情報に関する定義
    '----------------------------------------------------------------
    Public Const DATE_TIME_LENGTH As Integer = 7                    ' 日時の長さ(年/月/日(曜) 時:分:秒)
    Public Const START_TIME_LENGTH As Integer = 5                   ' スタートタイムの長さ(日/時/分/秒/ミリ秒)

    '----------------------------------------------------------------
    ' ステータスに関する定義
    '----------------------------------------------------------------
    Public Const STOP_LOGGING As UInteger = &H0                     ' ロギング停止
    Public Const RUNNING_LOGGING As UInteger = &H1                  ' ロギング実行中
    Public Const REACH_START_TIME As UInteger = &H2                 ' スタートタイムに達した
    Public Const PRE_START_TRIGPOS As UInteger = &H4                ' スタートトリガポジションに達する前
    Public Const WAIT_START_TRIG As UInteger = &H8                  ' スタートトリガ待ち
    Public Const DETECT_START_TRIG As UInteger = &H10               ' スタートトリガ検出
    Public Const PRE_STOP_TRIGPOS As UInteger = &H20                ' ストップトリガポジションに達する前
    Public Const WAIT_STOP_TRIG As UInteger = &H40                  ' ストップトリガ待ち
    Public Const DETECT_STOP_TRIG As UInteger = &H80                ' ストップトリガ検出

    '----------------------------------------------------------------
    ' エラーコードの定義
    '----------------------------------------------------------------
    Public Const SUCCESS As UInteger = &H0                          ' 異常なし
    Public Const ERR_SYSTEM As UInteger = &H1                       ' システムエラー
    Public Const ERR_NO_DEVICE As UInteger = &H2                    ' 使用可能なデバイスがありません
    Public Const ERR_IN_USE As UInteger = &H3                       ' 指定のデバイスは使用中です
    Public Const ERR_INVALID_DEVICENO As UInteger = &H4             ' 不正なデバイス識別番号です
    Public Const ERR_INVALID_ARGUMENT As UInteger = &H5             ' 不正な引数が指定されました
    Public Const ERR_INTERNAL As UInteger = &H6                     ' 内部エラーです
    Public Const ERR_TRANSFER_TIMEOUT As UInteger = &H7             ' データ転送がタイムアウトになりました
    Public Const ERR_TRANSFER_CANCEL As UInteger = &H8              ' データ転送がキャンセルされました
    Public Const ERR_TRANSFER_FAIL As UInteger = &H9                ' データ転送が失敗しました
    Public Const ERR_TRANSFER_SIZE As UInteger = &HA                ' データ転送のサイズが不正です
    Public Const ERR_HARDBUFF_OVERRUN As UInteger = &HB             ' ハードバッファがオーバーランしました
    Public Const ERR_SOFTBUFF_OVERRUN As UInteger = &HC             ' ソフトバッファがオーバーランしました
    Public Const ERR_DEVICE As UInteger = &HD                       ' デバイスエラーです
    Public Const ERR_WRAPDLL As UInteger = &HFFFF                   ' DLL のロードに失敗しました

    Public Const ERR_ANALOG_SETTING As UInteger = &H1               ' アナログに関する設定で不正な引数が指定されました
    Public Const ERR_DIGITAL_SETTING As UInteger = &H4              ' デジタルに関する設定で不正な引数が指定されました
    Public Const ERR_ALARM_SETTING As UInteger = &H5                ' アラームに関する設定で不正な引数が指定されました
    Public Const ERR_TRIGMODE_SETTING As UInteger = &H6             ' トリガモードに関する設定で不正な引数が指定されました
    Public Const ERR_EXTRIG_SETTING As UInteger = &H7               ' 外部トリガに関する設定で不正な引数が指定されました

    ' アナログに関する設定で不正な引数が指定されました
    Public Const ERR_ANALOG_CH_ENABLE As UInteger = &H10            ' チャンネル使用状態
    Public Const ERR_ANALOG_TYPE As UInteger = &H11                 ' タイプ
    Public Const ERR_ANALOG_RANGE As UInteger = &H12                ' レンジ
    Public Const ERR_ANALOG_BURNOUT As UInteger = &H13              ' バーンアウト
    Public Const ERR_ANALOG_COLDJUNCCOMP As UInteger = &H14         ' 冷接点補償
    Public Const ERR_ANALOG_SAMPLINGMODE As UInteger = &H15         ' サンプリングモード
    Public Const ERR_ANALOG_SAMPLINGTIME As UInteger = &H16         ' サンプリングタイム
    Public Const ERR_ANALOG_STARTTIME_ENABLE As UInteger = &H17     ' スタートタイム使用状態
    Public Const ERR_ANALOG_STARTTIME As UInteger = &H18            ' スタートタイム
    Public Const ERR_ANALOG_LOGGINGSIZE As UInteger = &H19          ' ロギングサイズ
    Public Const ERR_ANALOG_STARTTRIG_ENABLE As UInteger = &H1A     ' スタートトリガ使用状態
    Public Const ERR_ANALOG_STARTTRIG_CH As UInteger = &H1B         ' スタートトリガチャンネル
    Public Const ERR_ANALOG_STARTTRIG_MODE As UInteger = &H1C       ' スタートトリガモード
    Public Const ERR_ANALOG_STARTTRIG_SENSE As UInteger = &H1D      ' スタートトリガ検出モード
    Public Const ERR_ANALOG_STARTTRIG_LEVEL As UInteger = &H1E      ' スタートトリガレベル
    Public Const ERR_ANALOG_STARTTRIG_POS As UInteger = &H1F        ' スタートトリガポジション
    Public Const ERR_ANALOG_STOPTRIG_ENABLE As UInteger = &H20      ' ストップトリガ使用状態
    Public Const ERR_ANALOG_STOPTRIG_CH As UInteger = &H21          ' ストップトリガチャンネル
    Public Const ERR_ANALOG_STOPTRIG_MODE As UInteger = &H22        ' ストップトリガモード
    Public Const ERR_ANALOG_STOPTRIG_SENSE As UInteger = &H23       ' ストップトリガ検出モード
    Public Const ERR_ANALOG_STOPTRIG_LEVEL As UInteger = &H24       ' ストップトリガレベル
    Public Const ERR_ANALOG_STOPTRIG_POS As UInteger = &H25         ' ストップトリガポジション

    ' 外部トリガに関する設定で不正な引数が指定されました
    Public Const ERR_EXTRIG_SLOPE As UInteger = &H30                ' スロープ
    Public Const ERR_EXTRIG_FILTER As UInteger = &H31               ' フィルタ

    ' アラームに関する設定で不正な引数が指定されました
    Public Const ERR_ALARM_ANALOG_ENABLE As UInteger = &H40         ' アナログ使用状態
    Public Const ERR_ALARM_ANALOG_MODE As UInteger = &H41           ' アナログモード
    Public Const ERR_ALARM_ANALOG_SENSE As UInteger = &H42          ' アナログ検出モード
    Public Const ERR_ALARM_ANALOG_LEVEL As UInteger = &H43          ' アナログレベル
    Public Const ERR_ALARM_ANALOG_PORTOUT_ENABLE As UInteger = &H44 ' アナログポート出力状態
    Public Const ERR_ALARM_ANALOG_PORTOUT_CH As UInteger = &H45     ' アナログポート出力チャンネル
    Public Const ERR_ALARM_EXTRIG_ENABLE As UInteger = &H4A         ' 外部トリガ使用状態
    Public Const ERR_ALARM_EXTRIG_CH As UInteger = &H4B             ' 外部トリガチャンネル
    Public Const ERR_ALARM_EXTRIG_PORTOUT_ENABLE As UInteger = &H4C ' 外部トリガポート出力状態
    Public Const ERR_ALARM_EXTRIG_PORTOUT_CH As UInteger = &H4D     ' 外部トリガポート出力チャンネル

    ' デジタル出力に関する設定で不正な引数が指定されました
    Public Const ERR_DIGITALOUT_TYPE As UInteger = &H50             ' 出力タイプ
    Public Const ERR_DIGITALOUT_LEVEL As UInteger = &H51            ' 出力論理

    ' デバイスエラー
    Public Const ERR_FPGA_SETTING As UInteger = &H10                ' FPGA:設定処理が異常です
    Public Const ERR_RTC_INIT As UInteger = &H20                    ' RTC:イニシャライズ処理が異常です
    Public Const ERR_RTC_DATA_ACCESS As UInteger = &H21             ' RTC:データ転送アクセスが異常です
    Public Const ERR_RTC_DATA_SETTING As UInteger = &H22            ' RTC:データ設定が異常です
    Public Const ERR_RTC_TIME_SETTING As UInteger = &H23            ' RTC:時刻設定が異常です

    '----------------------------------------------------------------
    ' メッセージの定義
    '----------------------------------------------------------------
    Public Const WM_REMOVAL As Integer = 1                          ' デバイス抜去
    Public Const WM_INSERT As Integer = 2                           ' デバイス挿入
    Public Const WM_ERROR As Integer = 3                            ' データ取得スレッド内エラー

    '----------------------------------------------------------------
    ' その他の定義
    '----------------------------------------------------------------
    Public Const DEVICE_ID As UInteger = &H5310                     ' デバイスID
    Public Const DLL_NAME As String = "pas1000.dll"                 ' DLLファイル名

    '----------------------------------------------------------------
    ' デバイス情報構造体
    '----------------------------------------------------------------
    Structure TDEVINFO
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=VID_LENGTH)> _
        Dim bVendorID() As Byte                                     ' ベンダ識別文字列
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=PID_LENGTH)> _
        Dim bProductID() As Byte                                    ' プロダクト識別文字列
        Dim bFirmVer As Byte                                        ' ファームウェアのバージョン情報
        Dim bFpgaVer As Byte                                        ' FPGAのバージョン情報
        Dim bUsbMode As Byte                                        ' USB転送モード
    End Structure

    '----------------------------------------------------------------
    ' 動作モードパラメータ構造体
    '----------------------------------------------------------------
    ' トリガ機能構造体
    Structure TTRIG
        Dim dwEnable As UInteger                                    ' トリガ使用状態の設定
        Dim dwCh As UInteger                                        ' トリガチャンネルの設定
        Dim dwMode As UInteger                                      ' トリガモードの設定
        Dim dwSense As UInteger                                     ' トリガ検出モードの設定
        Dim dwLevel0 As UInteger                                    ' トリガレベル0の設定
        Dim dwLevel1 As UInteger                                    ' トリガレベル1の設定
        Dim dwPos As ULong                                          ' トリガポジションの設定
    End Structure

    ' アナログ機能構造体(チャンネル個別の設定)
    Structure TANALOGCH
        Dim dwChEnable As UInteger                                  ' アナログチャンネルの設定
        Dim dwType As UInteger                                      ' アナログタイプの設定
        Dim dwRange As UInteger                                     ' アナログ入力レンジの設定
        Dim dwBurnOut As UInteger                                   ' アナログバーンアウトの設定
        Dim dwColdJuncComp As UInteger                              ' アナログ冷接点補償の設定
    End Structure

    ' アナログ機能構造体
    Structure TANALOG
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ANALOG_NUM_CH)> _
        Dim Ch() As TANALOGCH                                       ' アナログチャンネル個別の設定
        Dim StartTrig As TTRIG                                      ' アナログスタートトリガの設定
        Dim StopTrig As TTRIG                                       ' アナログストップトリガの設定
        Dim dwSamplingMode As UInteger                              ' アナログサンプリングモードの設定
        Dim dwSamplingTime As UInteger                              ' アナログサンプリングタイムの設定
        Dim dwStartTimeEnable As UInteger                           ' アナログスタートタイム使用状態の設定
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=START_TIME_LENGTH)> _
        Dim dwStartTime() As UInteger                               ' アナログスタートタイムの設定
        Dim dwLoggingSize As ULong                                  ' アナログロギングサイズの設定
    End Structure

    ' 外部トリガ構造体
    Structure TEXTRIG
        Dim dwInSlope As UInteger                                   ' 入力スロープ
        Dim dwInFilter As UInteger                                  ' 入力フィルタ
    End Structure

    ' アラーム機能のアナログ構造体
    Structure TALARMANALOG
        Dim dwEnable As UInteger                                    ' アラーム使用状態の設定
        Dim dwMode As UInteger                                      ' アラームモードの設定
        Dim dwSense As UInteger                                     ' アラーム検出モードの設定
        Dim dwLevel0 As UInteger                                    ' アラームレベル0の設定
        Dim dwLevel1 As UInteger                                    ' アラームレベル1の設定
        Dim dwPortOutEnable As UInteger                             ' アラームポート出力状態の設定
        Dim dwPortOutCh As UInteger                                 ' アラームポート出力チャンネルの設定
    End Structure

    ' アラーム機能の外部トリガ構造体
    Structure TALARMEXTRIG
        Dim dwEnable As UInteger                                    ' アラーム使用状態の設定
        Dim dwCh As UInteger                                        ' アラームチャンネルの設定
        Dim dwPortOutEnable As UInteger                             ' アラームポート出力状態の設定
        Dim dwPortOutCh As UInteger                                 ' アラームポート出力チャンネルの設定
    End Structure

    ' アラーム機能構造体
    Structure TALARM
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ANALOG_NUM_CH)> _
        Dim Analog() As TALARMANALOG                                ' アナログのアラーム要因の設定
        Dim ExTrig As TALARMEXTRIG                                  ' 外部トリガのアラーム要因の設定
    End Structure

    ' デジタル出力構造体
    Structure TDIGITALOUT
        Dim dwType As UInteger                                      ' デジタル出力タイプの設定
        Dim dwLevel As UInteger                                     ' デジタル出力論理の設定
    End Structure

    ' 動作モード構造体
    Structure TSETMODE
        Dim Analog As TANALOG                                       ' アナログ機能
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=EXTRIG_NUM_CH)> _
        Dim ExTrig() As TEXTRIG                                     ' 外部トリガ機能
        Dim Alarm As TALARM                                         ' アラーム機能
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=DIGITALOUT_NUM_CH)> _
        Dim DigitalOut() As TDIGITALOUT                             ' デジタル出力
    End Structure

    '----------------------------------------------------------------
    ' チャンネルごとのステータス構造体
    '----------------------------------------------------------------
    ' ロギングステータス構造体
    Structure TLOGGINGSTS
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=ANALOG_NUM_CH)> _
        Dim dwAnalog As UInteger                                    ' アナログ
    End Structure

    ' チャンネルステータス構造体
    Structure TCHSTATUS
        Dim LoggingStatus As TLOGGINGSTS                            ' ロギングステータス
        Dim dwAlarmStatus As UInteger                               ' アラームステータス
    End Structure

    '----------------------------------------------------------------
    ' API
    '----------------------------------------------------------------
    ' 初期化処理
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000Initialize(ByVal hWnd As Integer, ByVal uWMBase As UInteger, ByVal TimeOut As Short) As Boolean
    End Function

    ' デバイスの検索
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000Create(ByRef pwDeviceNo As UShort) As Boolean
    End Function

    ' デバイスの開放
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000Close(ByVal wDeviceNo As UShort) As Boolean
    End Function

    ' 終了処理
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000Terminate() As Boolean
    End Function

    ' バージョン情報を取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetVersion(ByRef pdwDrvVer As UInteger, ByRef pdwDllVer As UInteger) As Boolean
    End Function

    ' デバイス情報を取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetDevInfo(ByVal wDeviceNo As UShort, ByVal pDevInfo As IntPtr) As Boolean
    End Function

    ' ユニットIDの設定
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000SetUnitId(ByVal wDeviceNo As UShort, ByVal dwUnitId As UInteger) As Boolean
    End Function

    ' ユニット情報を取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetUnitInfo(ByVal wDeviceNo As UShort, ByRef pdwUnitId As UInteger, ByRef pdwNumCh As UInteger) As Boolean
    End Function

    ' 時刻の設定
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000SetTime(ByVal wDeviceNo As UShort, ByVal pdwHostTime As IntPtr) As Boolean
    End Function

    ' 時刻を取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetTime(ByVal wDeviceNo As UShort, ByVal pdwFirmTime As IntPtr, _
                                          ByVal pdwFpgaTime As IntPtr, ByRef pdwAdjustStatus As UInteger) As Boolean
    End Function

    ' 動作モードの設定
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000SetMode(ByVal wDeviceNo As UShort, ByVal pSetMode As IntPtr) As Boolean
    End Function

    ' デジタル入力の読み込み
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000DigitalIn(ByVal wDeviceNo As UShort, ByRef pbDigitalInValue As Byte) As Boolean
    End Function

    ' デジタル出力の書き込み
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000DigitalOut(ByVal wDeviceNo As UShort, ByVal bDigitalOutValue As Byte) As Boolean
    End Function

    ' アラームのクリア
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000AlarmClear(ByVal wDeviceNo As UShort, ByVal dwNumCh As UInteger) As Boolean
    End Function

    ' ロギングの開始
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000StartLogging(ByVal wDeviceNo As UShort, ByVal dwStartMode As UInteger) As Boolean
    End Function

    ' ロギングの停止
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000StopLogging(ByVal wDeviceNo As UShort) As Boolean
    End Function

    ' チャンネル共通のステータスを取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetStatus(ByVal wDeviceNo As UShort, ByRef pdwHardNumData As UInteger, _
                                                ByRef pdwSoftNumData As UInteger, ByRef pdwLoggingStatus As UInteger, _
                                                ByRef pdwOverrunError As UInteger) As Boolean
    End Function

    ' チャンネルごとのステータスを取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetChStatus(ByVal wDeviceNo As UShort, ByVal pChStatus As IntPtr) As Boolean
    End Function

    ' ロギングデータを取得
    <DllImport(DLL_NAME)> _
    Public Shared Function Pas1000GetData(ByVal wDeviceNo As UShort, ByVal dwNumData As UInteger, ByVal pbData As IntPtr) As Boolean
    End Function

    ' エラーコードを取得
    <DllImportAttribute(DLL_NAME, SetLastError:=True)> _
    Public Shared Function Pas1000GetLastError(ByVal wDeviceNo As UShort) As UInteger
    End Function

    ''' <summary>
    ''' 初期化処理。
    ''' </summary>
    ''' <param name="hWnd">DLLが送出するメッセージを受け取るウィンドウのハンドル。</param>
    ''' <param name="uWMBase">送出するメッセージIDのベース値。</param>
    ''' <param name="TimeOut">USB 転送完了待ちのタイムアウト時間。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function Initialize(ByVal hWnd As Integer, ByVal uWMBase As UInteger, ByVal TimeOut As Short) As Boolean
        Try
            Initialize = Pas1000Initialize(hWnd, uWMBase, TimeOut)
        Catch e As Exception
            Initialize = False
        End Try
    End Function
    ''' <summary>
    ''' デバイスの検索。
    ''' </summary>
    ''' <param name="pwDeviceNo">使用するデバイスのデバイス識別番号を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function Create(ByRef pwDeviceNo As UShort) As Boolean
        Try
            Create = Pas1000Create(pwDeviceNo)
        Catch e As Exception
            Create = False
        End Try
    End Function
    ''' <summary>
    ''' デバイスの開放。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function Close(ByVal wDeviceNo As UShort) As Boolean
        Try
            Close = Pas1000Close(wDeviceNo)
        Catch e As Exception
            Close = False
        End Try
    End Function
    ''' <summary>
    ''' 終了処理。
    ''' </summary>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function Terminate() As Boolean
        Try
            Terminate = Pas1000Terminate()
        Catch e As Exception
            Terminate = False
        End Try
    End Function
    ''' <summary>
    ''' バージョン情報を取得。
    ''' </summary>
    ''' <param name="pdwDrvVer">ドライバのバージョン情報を格納する領域へのポインタ。</param>
    ''' <param name="pdwDllVer">DLLのバージョン情報を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetVersion(ByRef pdwDrvVer As UInteger, ByRef pdwDllVer As UInteger) As Boolean
        Try
            GetVersion = Pas1000GetVersion(pdwDrvVer, pdwDllVer)
        Catch e As Exception
            GetVersion = False
        End Try
    End Function
    ''' <summary>
    ''' デバイス情報を取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pDevInfo">デバイス情報を格納する構造体へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetDevInfo(ByVal wDeviceNo As UShort, ByRef pDevInfo As TDEVINFO) As Boolean
        Try
            Dim res As Boolean
            Dim ptr As IntPtr

            ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pDevInfo))    ' アンマネージメモリ確保
            Marshal.StructureToPtr(pDevInfo, ptr, False)            ' マーシャリング(マネージメモリ->アンマネージメモリ)
            res = Pas1000GetDevInfo(wDeviceNo, ptr)
            pDevInfo = CType(Marshal.PtrToStructure(ptr, GetType(TDEVINFO)), TDEVINFO)  ' マーシャリング(アンマネージメモリ->マネージメモリ)
            Marshal.FreeHGlobal(ptr)                                ' メモリを解放

            GetDevInfo = res
        Catch e As Exception
            GetDevInfo = False
        End Try
    End Function
    ''' <summary>
    ''' ユニットIDの設定。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="dwUnitId">ユニットID番号。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function SetUnitId(ByVal wDeviceNo As UShort, ByVal dwUnitId As UInteger) As Boolean
        Try
            SetUnitId = Pas1000SetUnitId(wDeviceNo, dwUnitId)
        Catch e As Exception
            SetUnitId = False
        End Try
    End Function
    ''' <summary>
    ''' ユニット情報を取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pdwUnitId">ユニットIDを格納する領域へのポインタ。</param>
    ''' <param name="pdwNumCh">ユニットに搭載されているアナログチャンネル数を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetUnitInfo(ByVal wDeviceNo As UShort, ByRef pdwUnitId As UInteger, ByRef pdwNumCh As UInteger) As Boolean
        Try
            GetUnitInfo = Pas1000GetUnitInfo(wDeviceNo, pdwUnitId, pdwNumCh)
        Catch e As Exception
            GetUnitInfo = False
        End Try
    End Function
    ''' <summary>
    ''' 時刻の設定。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pdwHostTime">日時を格納した領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function SetTime(ByVal wDeviceNo As UShort, ByRef pdwHostTime() As Integer) As Boolean
        Try
            Dim res As Boolean
            Dim size As Integer
            Dim ptr As IntPtr

            size = Marshal.SizeOf(pdwHostTime(LBound(pdwHostTime))) * pdwHostTime.Length
            ptr = Marshal.AllocHGlobal(size)                        ' アンマネージメモリ確保
            Marshal.Copy(pdwHostTime, 0, ptr, pdwHostTime.Length)   ' マネージメモリをアンマネージメモリにコピー
            res = Pas1000SetTime(wDeviceNo, ptr)
            Marshal.FreeHGlobal(ptr)                                ' メモリを解放

            SetTime = res
        Catch e As Exception
            SetTime = False
        End Try
    End Function
    ''' <summary>
    ''' 時刻を取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pdwFirmTime">ファーム時計の日時を格納する領域へのポインタ。</param>
    ''' <param name="pdwFpgaTime">FPGA時計の日時を格納する領域へのポインタ。</param>
    ''' <param name="pdwAdjustStatus">アジャストの状態を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetTime(ByVal wDeviceNo As UShort, ByRef pdwFirmTime() As Integer, _
                            ByRef pdwFpgaTime() As Integer, ByRef pdwAdjustStatus As UInteger) As Boolean
        Try
            Dim res As Boolean
            Dim size As Integer
            Dim ptrFirm As IntPtr
            Dim ptrFpga As IntPtr
            
            size = Marshal.SizeOf(pdwFirmTime(LBound(pdwFirmTime))) * pdwFirmTime.Length
            ptrFirm = Marshal.AllocHGlobal(size)                        ' アンマネージメモリ確保
            Marshal.Copy(pdwFirmTime, 0, ptrFirm, pdwFirmTime.Length)   ' マネージメモリをアンマネージメモリにコピー
            size = Marshal.SizeOf(pdwFpgaTime(LBound(pdwFpgaTime))) * pdwFpgaTime.Length
            ptrFpga = Marshal.AllocHGlobal(size)
            Marshal.Copy(pdwFpgaTime, 0, ptrFpga, pdwFpgaTime.Length)
            res = Pas1000GetTime(wDeviceNo, ptrFirm, ptrFpga, pdwAdjustStatus)
            Marshal.Copy(ptrFirm, pdwFirmTime, 0, pdwFirmTime.Length)   ' アンマネージメモリをマネージメモリにコピー
            Marshal.Copy(ptrFpga, pdwFpgaTime, 0, pdwFpgaTime.Length)
            Marshal.FreeHGlobal(ptrFirm)                                ' メモリを解放
            Marshal.FreeHGlobal(ptrFpga)

            GetTime = res
        Catch e As Exception
            GetTime = False
        End Try
    End Function
    ''' <summary>
    ''' 動作モードの設定。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pSetMode">設定情報を格納している構造体のポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function SetMode(ByVal wDeviceNo As UShort, ByVal pSetMode As TSETMODE) As Boolean
        Try
            Dim res As Boolean
            Dim ptr As IntPtr

            ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pSetMode))    ' アンマネージメモリ確保
            Marshal.StructureToPtr(pSetMode, ptr, False)            ' マーシャリング(マネージメモリ->アンマネージメモリ)
            res = Pas1000SetMode(wDeviceNo, ptr)
            Marshal.FreeHGlobal(ptr)                                ' メモリを解放

            SetMode = res
        Catch e As Exception
            SetMode = False
        End Try
    End Function
    ''' <summary>
    ''' デジタル入力の読み込み。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pbDigitalInValue">デジタル入力を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function DigitalIn(ByVal wDeviceNo As UShort, ByRef pbDigitalInValue As Byte) As Boolean
        Try
            DigitalIn = Pas1000DigitalIn(wDeviceNo, pbDigitalInValue)
        Catch e As Exception
            DigitalIn = False
        End Try
    End Function
    ''' <summary>
    ''' デジタル出力の書き込み。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="bDigitalOutValue">デジタル出力データ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function DigitalOut(ByVal wDeviceNo As UShort, ByVal bDigitalOutValue As Byte) As Boolean
        Try
            DigitalOut = Pas1000DigitalOut(wDeviceNo, bDigitalOutValue)
        Catch e As Exception
            DigitalOut = False
        End Try
    End Function
    ''' <summary>
    ''' アラームのクリア。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="dwNumCh">チャンネル番号。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function AlarmClear(ByVal wDeviceNo As UShort, ByVal dwNumCh As UInteger) As Boolean
        Try
            AlarmClear = Pas1000AlarmClear(wDeviceNo, dwNumCh)
        Catch e As Exception
            AlarmClear = False
        End Try
    End Function
    ''' <summary>
    ''' ロギングの開始。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="dwStartMode">開始モード。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function StartLogging(ByVal wDeviceNo As UShort, ByVal dwStartMode As UInteger) As Boolean
        Try
            StartLogging = Pas1000StartLogging(wDeviceNo, dwStartMode)
        Catch e As Exception
            StartLogging = False
        End Try
    End Function
    ''' <summary>
    ''' ロギングの停止。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function StopLogging(ByVal wDeviceNo As UShort) As Boolean
        Try
            StopLogging = Pas1000StopLogging(wDeviceNo)
        Catch e As Exception
            StopLogging = False
        End Try
    End Function
    ''' <summary>
    ''' チャンネル共通のステータスを取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pdwHardNumData">ハードバッファのロギングデータ数を格納する領域へのポインタ。</param>
    ''' <param name="pdwSoftNumData">ソフトバッファのロギングデータ数を格納する領域へのポインタ。</param>
    ''' <param name="pdwLoggingStatus">ロギングステータスを格納する領域へのポインタ。</param>
    ''' <param name="pdwOverrunError">オーバーランエラーを格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetStatus(ByVal wDeviceNo As UShort, ByRef pdwHardNumData As UInteger, _
                              ByRef pdwSoftNumData As UInteger, ByRef pdwLoggingStatus As UInteger, _
                              ByRef pdwOverrunError As UInteger) As Boolean
        Try
            GetStatus = Pas1000GetStatus(wDeviceNo, pdwHardNumData, pdwSoftNumData, pdwLoggingStatus, pdwOverrunError)
        Catch e As Exception
            GetStatus = False
        End Try
    End Function
    ''' <summary>
    ''' チャンネルごとのステータスを取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="pChStatus">ステータス情報を格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetChStatus(ByVal wDeviceNo As UShort, ByRef pChStatus As TCHSTATUS) As Boolean
        Try
            Dim res As Boolean
            Dim ptr As IntPtr

            ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pChStatus))   ' アンマネージメモリ確保
            Marshal.StructureToPtr(pChStatus, ptr, False)           ' マーシャリング(マネージメモリ->アンマネージメモリ)
            res = Pas1000GetChStatus(wDeviceNo, ptr)
            pChStatus = CType(Marshal.PtrToStructure(ptr, GetType(TCHSTATUS)), TCHSTATUS)   ' マーシャリング(アンマネージメモリ->マネージメモリ)
            Marshal.FreeHGlobal(ptr)                                ' メモリを解放

            GetChStatus = res
        Catch e As Exception
            GetChStatus = False
        End Try
    End Function
    ''' <summary>
    ''' ロギングデータを取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <param name="dwNumData">取得するデータ数。</param>
    ''' <param name="pbData">ロギングデータを格納する領域へのポインタ。</param>
    ''' <returns>成功 : True、失敗 : False</returns>
    Public Function GetData(ByVal wDeviceNo As UShort, ByVal dwNumData As UInteger, ByRef pbData() As Byte) As Boolean
        Try
            Dim res As Boolean
            Dim size As Integer
            Dim ptr As IntPtr

            size = Marshal.SizeOf(pbData(LBound(pbData))) * pbData.Length
            ptr = Marshal.AllocHGlobal(size)            ' アンマネージメモリ確保
            Marshal.Copy(pbData, 0, ptr, pbData.Length) ' マネージメモリをアンマネージメモリにコピー
            res = Pas1000GetData(wDeviceNo, dwNumData, ptr)
            Marshal.Copy(ptr, pbData, 0, pbData.Length) ' アンマネージメモリをマネージメモリにコピー
            Marshal.FreeHGlobal(ptr)                    ' メモリを解放

            GetData = res
        Catch e As Exception
            GetData = False
        End Try
    End Function
    ''' <summary>
    ''' エラーコードを取得。
    ''' </summary>
    ''' <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
    ''' <returns>エラーコード。</returns>
    Public Function GetLastError(ByVal wDeviceNo As UShort) As UInteger
        Try
            GetLastError = Pas1000GetLastError(wDeviceNo)
        Catch e As Exception
            GetLastError = ERR_WRAPDLL
        End Try
    End Function

End Class
